
#ifndef BONDPOTENTIAL_H
#define BONDPOTENTIAL_H

#include "rumd/Potential.h"

class MoleculeData; class Sample;

///////////////////////////////////////////////////////
// Abstract BondPotential class
///////////////////////////////////////////////////////

class BondPotential : public Potential{
 public:
  MoleculeData *M;
  Sample *S;
};

///////////////////////////////////////////////////////
// Specialized bond potentials
///////////////////////////////////////////////////////

class BondHarmonic : public BondPotential{
 private:
  BondHarmonic(const BondHarmonic&);
  BondHarmonic& operator=(const BondHarmonic&);

 public:
  BondHarmonic(MoleculeData *, Sample *);
  void CalcF(bool initialize, bool calc_stresses, bool calc_temp, bool calc_bs);
  double GetPotentialEnergy();
};

class BondFENE : public BondPotential{
 private:
  BondFENE(const BondFENE&);
  BondFENE& operator=(const BondFENE&);
  
 public:
  BondFENE(MoleculeData *, Sample *);
  void CalcF(bool initialize, bool calc_stresses, bool calc_temp, bool calc_bs);
  double GetPotentialEnergy();
};

///////////////////////////////////////////////////////
// Kernel prototypes
///////////////////////////////////////////////////////

template <int stress, int energy, bool initialize, class Simbox> __global__ void Harmonic( uint1 *blist, float4 *r, float4 *f, float4 *w, 
											   unsigned tpb, unsigned max_num_bonds, unsigned npart, 
											   unsigned virt_part, Simbox *simbox, float *simboxpointer, 
											   int1 *btlist, float2 *bplist, int1 *btlist_int, bool shape );

template <int stress, int energy, bool initialize, class Simbox> __global__ void FENE( uint1 *blist, float4 *r, float4 *f, float4 *w,  
										       unsigned tpb, unsigned max_num_bonds, unsigned npart, 
										       unsigned virt_part, Simbox *simbox, float *simboxpointer, 
										       int1 *btlist, int1 *btlist_int, bool shape );

#endif // BONDPOTENTIAL_H
