#ifndef ENERGIESMETADATA_H
#define ENERGIESMETADATA_H

#include <string>
#include <map>
#include <cassert>
#include <cstdlib>

#include "rumd/LogLin.h"
#include "rumd/RUMD_Error.h"

class EnergiesMetaData 
{
public:
  EnergiesMetaData():
    rumd_energies_ioformat(0),
    precision(6),
    writeInterval(100),
    usingAtomicVirPress(true),
    has_logLin(false),
    timeStepIndex(0),
    logLin(),
    options(),
    fileStr()
  {
    options["potentialEnergy"] = true;
    fileStr["potentialEnergy"] = "pe";
    options["kineticEnergy"] = true;
    fileStr["kineticEnergy"] = "ke";
    options["virial"] = true;
    fileStr["virial"] = "W";
    options["totalEnergy"] = true;
    fileStr["totalEnergy"] = "Etot";
    options["temperature"] = true;
    fileStr["temperature"] = "T";
    options["pressure"] = true;
    fileStr["pressure"] = "p";	
    options["volume"] = false;
    fileStr["volume"] = "V";
    options["thermostat_Ps"] = false;
    fileStr["thermostat_Ps"] = "Ps";
    options["enthalpy"] = false;
    fileStr["enthalpy"] = "H";
    options["potentialVirial"] = false;
    fileStr["potentialVirial"] = "pot_W";
    options["constraintVirial"] = false;
    fileStr["constraintVirial"] = "con_W";
    options["stress_xx"] = false;
    fileStr["stress_xx"] = "sxx";
    options["stress_yy"] = false;
    fileStr["stress_yy"] = "syy";
    options["stress_zz"] = false;
    fileStr["stress_zz"] = "szz";
    options["stress_yz"] = false;
    fileStr["stress_yz"] = "syz";
    options["stress_xz"] = false;
    fileStr["stress_xz"] = "sxz";
    options["stress_xy"] = false;
    fileStr["stress_xy"] = "sxy";
    options["v_com_x"] = false;
    fileStr["v_com_x"] = "v_com_x";
    options["v_com_y"] = false;
    fileStr["v_com_y"] = "v_com_y";
    options["v_com_z"] = false;
    fileStr["v_com_z"] = "v_com_z";
    options["configTemperature"] = false;
    fileStr["configTemperature"] = "configT";
    options["laplacian"] = false;
    fileStr["laplacian"] = "laplacian";
    options["simulationDisplacementLength"] = false;
    fileStr["simulationDisplacementLength"] = "dispLength";
    options["instantaneousTimeStepSq"] = false;
    fileStr["instantaneousTimeStepSq"] = "dt^2";
    options["euclideanLength"] = false;
    fileStr["euclideanLength"] = "eclLength";
 }
  void Set(const std::string& key, bool val) {
    if(options.find(key) == options.end())
      throw RUMD_Error("EnergiesMetaData","Set",std::string("Unrecognized key:") + key);
    options[key] = val;
  }
  bool Get(const std::string& key) {
    if(options.find(key) == options.end())
  throw RUMD_Error("EnergiesMetaData","Get",std::string("Unrecognized key:") + key);
    return options[key];
  }
  std::map<std::string, bool>::const_iterator Start() const {return options.begin();}
  std::map<std::string, bool>::const_iterator End() const {return options.end();}
  const std::string& GetFileString(const std::string& key) {return fileStr[key];}

  unsigned int rumd_energies_ioformat;
  int precision;
  int writeInterval;
  bool usingAtomicVirPress;
  bool has_logLin;
  unsigned int timeStepIndex;
  LogLin logLin;
  
  // the function which parses the comment line for configuration files
  void ReadMetaData(char *commentLine, int verbose);

  std::map<std::string, bool> options;
  std::map<std::string, std::string> fileStr;
};

#endif // ENERGIESMETADATA_H
