#ifndef MOLECULEDATA_H
#define MOLECULEDATA_H

#include "rumd/Sample.h"
#include "rumd/SimulationBox.h"
#include "rumd/PairPotential.h"
#include "rumd/ConstraintPotential.h"
#include "rumd/BondPotential.h"

class MoleculeData {
  friend class BondHarmonic;
  friend class BondFENE;

 private:
  Sample *S;                // The sample class 

  RectangularSimulationBox *SB;

  ConstraintPotential* constraintPotential;
 
  unsigned num_mol;         // Number of molecules 
  unsigned max_num_uau;     // Maximum nuau per molecule
  unsigned max_num_bonds;   // Maximum bonds per particle
  unsigned num_btypes;      // Number of bonding types 

  unsigned flagHarmonic;    // Harmonic bond class instantiated
  unsigned flagFENE;        // FENE bond class instantiated

 public:
  uint1 *h_blist;           // Bonding list 
  uint1 *d_blist; 
  unsigned lblist; 

  int1 *h_btlist;           // Bond type list 
  int1 *d_btlist;        
  unsigned lbtlist;            

  int1 *h_btlist_int;        // Internal bondlist
  int1 *d_btlist_int; 

  float2 *h_bplist;         // Bond type parameter list 
  float2 *d_bplist; 
 
  int1 *h_mlist;            // Molecule list 
  int1 *d_mlist;            // (Convenient to have 
  unsigned lmlist;          // for data analysis) 

  dim3 threads_per_block;   // GPU specifics
  dim3 num_blocks;

  // helper functions for ReadTopology
  void AddmlistEntry(unsigned *, unsigned, unsigned, unsigned);
  void AddblistEntry(unsigned , unsigned); 
  void AddbtlistEntry(unsigned *, unsigned, unsigned, unsigned);

  void CheckRead(unsigned, unsigned, unsigned);

  MoleculeData(const MoleculeData&); // no default copyconstruction
  MoleculeData& operator=(const MoleculeData&); // no default assignment

  void GetInfoFromTopology(const char *str);

 public:

  MoleculeData(Sample *, const char *);
  ~MoleculeData(void);

  BondHarmonic *HarmBond;  
  BondFENE *FENEBond;

  void ReadTopology(const char *);
  
  void PrintLists(void);
  void PrintLists(unsigned i);
  void PrintMlist(void);

  // Set methods.
  void SetBondHarmonic(unsigned, float, float);
  void SetBondFENE(unsigned);
  void SetBondConstraint(unsigned type, float lbond);

  void SetExclusionBond(PairPotential *potential, unsigned etype);
  void SetExclusionMolecule(PairPotential *potential, unsigned molindex);
  void SetExclusionMolecule(PairPotential *potential);

  // Get methods.
  unsigned GetNumberOfMolecules(){ return num_mol; };
};

#endif // MOLECULEDATA_H
