#ifndef POTENTIAL_H
#define POTENTIAL_H

#include "rumd/KernelPlan.h"
#include <string>
#include "rumd/RUMD_Error.h"

class Sample;
class ParticleData;
class SimulationBox;

class Potential{
 public:
  Potential();
  virtual ~Potential() {}
  virtual void CalcF(bool initialize, bool calc_stresses, bool calc_temp, bool calc_bs) = 0;
  virtual double GetPotentialEnergy() { return 0.; }
  virtual void SetSampleParameters(KernelPlan& kPlan, ParticleData* pd, SimulationBox* simBox, float4* d_misc );
  virtual void SetVerbose(bool vb) { verbose = vb; }
  std::string GetID_String() const { return ID_String; }
  void SetID_String(const std::string& set_ID_String) {ID_String = set_ID_String;}
  virtual Potential* Copy() {throw RUMD_Error("Potential","Copy","Not implemented yet");}
 protected:
  KernelPlan kp;
  ParticleData* particleData;
  SimulationBox* simBox;
    
  unsigned int nParticles;

  float4* d_r;
  float4* d_f;
  float4* d_w; 
  float4* d_sts;
  float4* d_misc; 

  bool verbose;
  
 private:
  Potential(const Potential&); // no default copy constructor
  Potential& operator=(const Potential&); // no default assignment operator
  std::string ID_String;
};

#endif // POTENTIAL_H
